#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include "generic.h"
#include "audioproc.h"

extern "C" {
#include "mpeg.h"
}

class Mp3Channel {
private:

  int spawn();

public:

  Mp3Channel();
  ~Mp3Channel();

  short gotin[BUF_SIZE];
  int sample_rate; /* must be setted when the channel is created */

  int stream[2];
  pid_t pid;
  short *inbuf; /* global inbuf for raw input - shared by all channels */
  char *name;
  float volume;
  float speed;
  int rate;
  int channels;
  bool end;
  bool on;
  char playlist[MAX_PLAYLIST_ENTRY][MAX_PATH_SIZE];
  int playlist_len;

  /* this must be setted if dsp is opened to let close the fd
     for the forked mpg123 */
  int *dsp;
  
  void start(int plpos);
  void stop();
  void ends();

  unsigned int get_audio();
  void get_playlist(char *plfile);
};

/* ----- MP3 CHANNEL ----- */

Mp3Channel::Mp3Channel() {
  volume = 1.0;
  speed = 1.0;
  rate = 0;
  channels = 0;
  on = false;
  end = false;
  stream[0] = 0;
  stream[1] = 0;
  pid = 0;
}

Mp3Channel::~Mp3Channel() {
  if(on)
    stop();
}

void Mp3Channel::start(int plpos) {
  if(pid) stop();
  name = &playlist[plpos][0];
  spawn();
  on = true;
  end = false;
}

void Mp3Channel::stop() {
  if(pid>0) {

    if(!end)
      kill(pid,3);

    waitpid(pid,NULL,0x0);
    
    close(stream[0]);
    close(stream[0]);
    
    on = false;
    end = false;
    pid = 0;
    stream[0] = 0;
    stream[1] = 1;
  } else {
    on = false;
    end = false;
  }

  memset(gotin,0,BUF_SIZE);
}

int Mp3Channel::spawn() {
  int strm[2], trate, tchannels;
  pid_t mypid;

  trate=format_of(name,&tchannels);
  rate=trate;
  channels=tchannels;

  pipe(strm);
  mypid=fork();
  if(mypid) {
    pid = mypid;
    stream[0] = strm[0];
    stream[1] = strm[1];
    return(0);
  } else {
    close(strm[0]);
    close(1);
    dup(strm[1]);
    close(strm[1]);

    /* QUAAAAAA */
    close(*dsp);
    execlp("mpg123","mpg123","-sq"/*,"-n 300"*/,name,NULL);
  }
  return -1;
}

void Mp3Channel::ends() {

  kill(pid,3); /* just to be sure */

  waitpid(pid,NULL,0x0);
  
  close(stream[0]);
  close(stream[1]);
  
  on = false;
  end = false;
  pid = 0;
  stream[0] = 0;
  stream[1] = 0;
}

unsigned int Mp3Channel::get_audio() {
  int i;
  unsigned int num_samples = 0;
  int opt;

  //  if(on) {

  num_samples=(int)((rate*BUF_SIZE*speed)/(sample_rate<<1)); /* for mono would be /(sample_rate) */
  
  opt = (num_samples*channels)<<1;
  
  i=read(stream[0],inbuf,opt);
  
  /* if we reach the end of the stream then ...
     Ok. It's a bug. it will be listed into the known bugs
     and it will be hopefully fixed. */
  if(i<opt) {
    /* for padding: for(;i<opt;i++) inbuf[i]=0; */
    end=true;
  }
  
  /* now convert the audio into something useful */
  convert_audio(inbuf,gotin,channels,2,num_samples,BUF_SIZE>>1);
  /* for mono would be:
     convert_audio(inbuf,stream_channel[ch].inbuf,stream_channel[ch].channels,1,num_samples,BUF_SIZE); */
  
  return num_samples;
} /* get_audio */

void Mp3Channel::get_playlist(char *plfile) {
  FILE *plist;
  int c = 0;
  char *p;

  plist = fopen(plfile,"r");
  if(plist == NULL) {
    fprintf(stderr,"Playlist file not found\n");
    fprintf(stderr,"to create a playlist file just do:\n");
    fprintf(stderr,"find /your/mp3/dir -name '*.mp3' -print >playlist\n");
    exit(0);
  }

  while (fgets(&playlist[c][0],MAX_PATH_SIZE,plist) != NULL) {
    p = &playlist[c][0];
    while(*p!='\n') p++;
    *p = '\0';
    c++;
  }
  fclose(plist);
  
  playlist_len = c;

}

/* ----- MP3 CHANNEL ----- */
