#include <iostream.h>
#include <getopt.h>
#include <string.h>
#include <unistd.h>

#include "generic.h"
#include "jmixer.h"
#include "netlistener.h"
#include "scheduler.h"
#include "libsync/synchronous.h"

/* ------------------- command line stuff --------------------------- */

char *version = "MuSE v0.3.0 - (c)2000 MuSE development team\n";

char *help =
"Usage: muse [options]\n\n"
"Generic options\n"
"-h --help\tthis help\n"
"-v --version\tversion information\n"
"-d --debug\tturn debug informations on\n"
"-D --daemon\tdaemon mode: do not spawn amuse local interface"
"-l --playlist\tuse filename as playlist (default ./playlist)\n"
"\n"
"--\n"
"for any further information refer to http://muse.dyne.org\n";

static const struct option long_options[] = {
  { "help", no_argument, NULL, 'h' },
  { "version", no_argument, NULL, 'v' },
  { "debug", no_argument, NULL, 'd' },
  { "daemon", no_argument, NULL, 'D' },
  { "playlist", required_argument, NULL, 'l' },
  { 0, 0, 0, 0 }
};

char *short_options = "hvdDl:";

/* misc settings */
bool debug = false;
bool daemon_mode = false;
char playlist[256];

void take_args(int argc, char **argv) {
  int res;

  strcpy(playlist,"./playlist");

  do {
    res = getopt_long(argc, argv, short_options, long_options, NULL);

    switch(res) {
    case 'h':
      fprintf(stderr,"%s",version);
      fprintf(stderr,"%s",help);
      exit(0);
      break;
    case 'v':
      fprintf(stderr,"%s",version);
      exit(0);
      break;
    case 'd':
      debug = true;
      break;
    case 'D':
      daemon_mode = true;
      break;
    case 'l':
      strncpy(playlist,optarg,256);
      break;
    }
  } while(res > 0);

}

int main(int argc, char **argv) {

  take_args(argc, argv);

  /* change those for having the stderr flodded by
     the libsync threaded kernel debugging messages */
  LTI::Kernel::trace = false;
  LTI::Kernel::debug = false;

  fprintf(stderr,"%sstarting engine...\n",version);

  Stream_mixer *mix = new Stream_mixer();

  if(!mix->open_soundcard()) {
    fprintf(stderr,"exiting.\n");
    exit(1);
  }

  /* we are still not implementing single channel playlists

     int c;
     for(c=0;c<MAX_CHANNELS;c++)
     mix->set_playlist(c, playlist);
     
     so we put a duplicated memory wasting unique playlist equal for all channels
     looking forward to the next release for the final playlist solution...
  */
  int c;
  for(c=0;c<MAX_CHANNELS;c++)
    mix->mp3ch[c].get_playlist(playlist);

  mix->start();

  Net_listener *net = new Net_listener(LISTENPORT,mix);

  net->start();

  /* spawn tcl/tk interface */
  if(!daemon_mode) {
    pid_t amuse;
    amuse = fork();
    if(!amuse) {
      fprintf(stderr,"Spawning the A-MuSE interface\n");
      execlp("./amuse","amuse",NULL);
      exit(0);
    }
  }

  net->join();
  mix->join();

  fprintf(stderr,"cya!\n");

  exit(0);
}
