#include <iostream.h>
#include <stdio.h>
#include <string.h>

#include "generic.h"

extern "C" {
#include "libshout/shout.h"
#include "lame/lame.h"
}

class IcecastOut {
private:
  shout_conn_t ice_conn;
  lame_global_flags enc_flags;

  char ip[20];
  int port;
  char pass[64];
  char mountpoint[64];
  int bps;
  bool initialized;

public:
  IcecastOut();

  bool start();
  void stop();
  void set(char *nip, int nport, char *npass, char *nmountpoint, int nbps);
  bool send(short *sendbuf, long len);
  void init_encoder(int bps);
  int encode(short int *pcm,int num,int *dest,int size);
  
  bool running;

  ~IcecastOut();
};

IcecastOut::IcecastOut() {

  initialized = false;
  running = false;

}

IcecastOut::~IcecastOut() {
  if(running)
    stop();
}

bool IcecastOut::start() {

  if(initialized&&!running) {
    shout_init_connection(&ice_conn);
    ice_conn.ip = ip;
    ice_conn.port = port;
    ice_conn.password = pass;
    ice_conn.mount = mountpoint;
    ice_conn.bitrate = bps*1024;
    
    fprintf(stderr,"Contacting icecast server ...",ice_conn.ip,port);
    
    if(shout_connect(&ice_conn)) {
      fprintf(stderr," done\n",mountpoint);
      running = true;
    } else {
      fprintf(stderr," ops!\nunable to connect or reach host\n");
      running = false;
    }
    return(running);
  } else
    return(false); /* icecast connection not initialized or already running */
}

void IcecastOut::stop() {
  if(running) {
    shout_disconnect(&ice_conn);
    running = false;
  }
}

void IcecastOut::set(char *nip, int nport, char *npass, char *nmountpoint, int nbps) {
  if(running)
    stop();

  strncpy(ip,nip,20);
  port = nport;
  strncpy(pass,npass,64);
  strncpy(mountpoint,nmountpoint,64);
  bps = nbps;

  initialized = true;
}

bool IcecastOut::send(short *sendbuf, long len) {
  if(running) {
    int res;
    res = shout_send_data(&ice_conn, (unsigned char *)sendbuf, len);
    if(!res) {
      fprintf(stderr,"Send error: %i\n",ice_conn.error);
      return(false);
    }
    shout_sleep(&ice_conn);
    return(res);
  } else
    return(false);
}

void IcecastOut::init_encoder(int bps) {
  
  fprintf(stderr,"Initializing mp3 encoder engine to %u ...",bps);

  lame_init(&enc_flags);
  enc_flags.num_channels = 1; /* the coming out stream is hardcoded stereo */
  enc_flags.in_samplerate = SAMPLE_RATE;
  enc_flags.brate = bps;
  enc_flags.mode = 1; /* jstereo */
  enc_flags.quality = QUALITY;
  lame_init_params(&enc_flags);

  fprintf(stderr," done\n");
}

int IcecastOut::encode(short int *pcm,int num,int *dest,int size) {
  int res;
  
  res = lame_encode_buffer_interleaved(&enc_flags, pcm, num, (char *) dest, size);
  return(res);
}
