/* scheduler.cpp - part of MuSE
   (c)2000 Denis Roio aka jaromil
   see COPYING for licensing stuff

   the format commands are stored in the queue file is:
   SECS.NANOSECS\tCMD\tARGS... */

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "jmixer.h"
#include "libsync/synchronous.h"

class Scheduler: public LTI::Object {
private:
  Stream_mixer *mixer;
  
  FILE *queuefd;
  timespec timelap;
  char nextevent[255];
  char *command;
  bool endqueue;
  char *p, *pp; /* parsing pointers */
  
public:
  Scheduler(char *queue_file, Stream_mixer *mix);

  void run();
  ~Scheduler();
};

Scheduler::Scheduler(char *queue_file, Stream_mixer *mix) {

  queuefd = fopen(queue_file, "r");
  if(queuefd!=NULL)
    endqueue = false;
  else
    endqueue = true;
  
  /* Initialization as a synchronous object */
  synchronous.registerObjectName("Scheduler");

  mixer = mix;

}

void Scheduler::run() {
  cerr << selfName() << " running on thread " << (int)pthread_self() << "\n";

  do {
    
    if(fgets(nextevent,255,queuefd)!=NULL) {
      fprintf(stderr,"SCHED: %s",nextevent);
      /* got it, let's see how long we must wait */
      p = &nextevent[0];
      while(*p!='.') p++;
      *p = '\0';
      timelap.tv_sec = atoi(nextevent);
      p++; pp = p;
      while(*p!='\t') p++;
      *p = '\0';
      timelap.tv_nsec = atol(pp);
      p++; command = p;
    } else
      endqueue = true;

    if(!endqueue) {
      fprintf(stderr,"SCHED: going to wait for %u secs %lu nsecs\n",timelap.tv_sec, timelap.tv_nsec);
      nanosleep(&timelap,NULL);
      
      fprintf(stderr,"SCHED: now it comes\n");

      /* executes the command */
      
      /* SET CHANNEL */
      if(strncmp(command,"301",3)==0) {

	int fnum, ch;
	float speed, vol;
	p = pp = command+4;
	while(*p!='\t') p++; *p = '\0';
	fnum = atoi(pp);
	p++; pp = p;
	while(*p!='\t') p++; *p = '\0';
	ch = atoi(pp);
	p++; pp = p;
	while(*p!='\t') p++; *p = '\0';
	speed = atof(pp);
	p++; pp = p;
	while(*p!='\n') p++; *p = '\0';
	vol = atof(pp);

	fprintf(stderr,"SCHED: got [set channel %u file %u vol %f speed %f]\n", ch, fnum, vol, speed);
	
	synchronous.select.call(0, mixer, Stream_mixer::id_set_channel);
	synchronous.wait();
	mixer->set_channel(ch,fnum);
	
	if(vol!=1.0) {
	  synchronous.select.call(0, mixer, Stream_mixer::id_set_volume);
	  synchronous.wait();
	  mixer->set_volume(ch,vol);
	}
	
	if(speed!=1.0) {
	  synchronous.select.call(0, mixer, Stream_mixer::id_set_speed);
	  synchronous.wait();
	  mixer->set_speed(ch,speed);
	}
      }
      /* -------- */
      
      /* STOP CHANNEL */
      if(strncmp(command,"302",3)==0) {
	int ch;
	p = pp = command+4;
	while(*p!='\n') p++; *p = '\0';
	ch = atoi(pp);
	
	synchronous.select.call(0, mixer, Stream_mixer::id_stop_channel);
	synchronous.wait();
	mixer->stop_channel(ch);
      }
      
      /* PAUSE CHANNEL */
      if(strncmp(command,"303",3)==0) {
	int ch;
	p = pp = command+4;
	while(*p!='\n') p++; *p = '\0';
	ch = atoi(pp);
	
	synchronous.select.call(0, mixer, Stream_mixer::id_pause_channel);
	synchronous.wait();
	mixer->pause_channel(ch);
      }
      
      /* START CHANNEL */
      if(strncmp(command,"304",3)==0) {
	int ch;
	p = pp = command+4;
	while(*p!='\n') p++; *p = '\0';
	ch = atoi(pp);
	
	synchronous.select.call(0, mixer, Stream_mixer::id_resume_channel);
	synchronous.wait();
	mixer->resume_channel(ch);
      }
      
      /* SET VOLUME */
      if(strncmp(command,"306",3)==0) {
	int ch;
	float vol;
	p = pp = command+4;
	while(*p!='\t') p++; *p = '\0';
	ch = atoi(pp);
	p++; pp = p;
	while(*p!='\n') p++; *p = '\0';
	vol = atof(pp);
	
	synchronous.select.call(0, mixer, Stream_mixer::id_set_volume);
	synchronous.wait();
	mixer->set_volume(ch,vol);
      }
    } /* if(!endqueue) */
  } while(!endqueue);
}

Scheduler::~Scheduler() {
  fclose(queuefd);
}
